#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Converted from MQL by Elliott Wave 11/06/2008
    /// </summary>
    [Description("Instantaneous Trend Indicator (iTrend): Trend indicator with nearly zero lag and about the same smoothing as EMA. Trade signals are generated by crossing of the trigger line and iTrend line. Original Code for Tradestation from the book of John F. Ehlers 'Cybernetic Analysis for Stocks and Futures' ")]
    public class iTrend : Indicator
    {
        #region Variables
        // Wizard generated variables
            private double alpha = 0.07; // Default setting for Alpha
		
			private int NumOfBars=0;
			private int PeriodsP=8;
		
			private DataSeries smooth;
        	private DataSeries  trend;
        	private DataSeries  trigger;
			
			
			
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            
			Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "TrendPlot"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "TriggerPlot"));
            
			
			trend = new DataSeries(this);
            trigger = new DataSeries(this);
			smooth  = new DataSeries(this);
            
			
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {

		
			if (CurrentBar==0)
			{
				smooth.Set(0);
				trigger.Set(0);
            	trend.Set(0);
				TrendPlot.Set(0);
				TriggerPlot.Set(0);
				
				return;
			}
			
			int Per1=PeriodsP;
			int Per2=PeriodsP;
			
			
			if(CurrentBar <= 7)
				return;
      
			double SmoothTemp = (Median[0] + 2* Median[1] + 2* Median[2] + Median[3])/6;
			smooth.Set(SmoothTemp);
			
			
			double ItrendTemp = (alpha - alpha*alpha/4)* smooth[0]+0.5*alpha*alpha*smooth[1]-(alpha-0.75*alpha*alpha)*smooth[2]+2*(1-alpha)*trend[1]-(1-alpha)*(1-alpha)*trend[2];
			//double ItrendTemp = (1 - .5 * alpha)* (1 - .5*alpha)*(smooth[0] - 2*smooth[1] + smooth[2]) + 2*(1 - alpha)* trend[1] - (1 - alpha)*(1 - alpha)*trend[2];
			trend.Set(ItrendTemp);
			
			trigger.Set(trend[1]);
			
			TrendPlot.Set(trend[0]);
			TriggerPlot.Set(trigger[0]);
			
			
			/*
		Price((H+L)/2),
		alpha(.07);
		Vars:
		Smooth(0),
		Itrend(0);
		Smooth = (Price + 2*Price[1] + 2*Price[2] + Price[3])/6;
		Itrend = (1 - .5*alpha)*(1 - .5*alpha)*(Smooth-2*Smooth[1] + Smooth[2]) + 2*(1 - alpha)*Itrend[1] - (1 - alpha)*(1 - alpha)*Itrend[2];
		If currentbar < 7 
			then Itrend = (Price - 2*Price[1]+ Price[2]) / 4;
		Plot1(Itrend, "Itrend");
		Plot2(Itrend[1], "Trigger");
		*/ 
					
			
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries TriggerPlot
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries TrendPlot
        {
            get { return Values[1]; }
        }

        [Description("")]
        [Category("Parameters")]
        public double Alpha
        {
            get { return alpha; }
            set { alpha = Math.Max(0, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private iTrend[] cacheiTrend = null;

        private static iTrend checkiTrend = new iTrend();

        /// <summary>
        /// Instantaneous Trend Indicator (iTrend): Trend indicator with nearly zero lag and about the same smoothing as EMA. Trade signals are generated by crossing of the trigger line and iTrend line. Original Code for Tradestation from the book of John F. Ehlers 'Cybernetic Analysis for Stocks and Futures' 
        /// </summary>
        /// <returns></returns>
        public iTrend iTrend(double alpha)
        {
            return iTrend(Input, alpha);
        }

        /// <summary>
        /// Instantaneous Trend Indicator (iTrend): Trend indicator with nearly zero lag and about the same smoothing as EMA. Trade signals are generated by crossing of the trigger line and iTrend line. Original Code for Tradestation from the book of John F. Ehlers 'Cybernetic Analysis for Stocks and Futures' 
        /// </summary>
        /// <returns></returns>
        public iTrend iTrend(Data.IDataSeries input, double alpha)
        {
            checkiTrend.Alpha = alpha;
            alpha = checkiTrend.Alpha;

            if (cacheiTrend != null)
                for (int idx = 0; idx < cacheiTrend.Length; idx++)
                    if (Math.Abs(cacheiTrend[idx].Alpha - alpha) <= double.Epsilon && cacheiTrend[idx].EqualsInput(input))
                        return cacheiTrend[idx];

            iTrend indicator = new iTrend();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Alpha = alpha;
            indicator.SetUp();

            iTrend[] tmp = new iTrend[cacheiTrend == null ? 1 : cacheiTrend.Length + 1];
            if (cacheiTrend != null)
                cacheiTrend.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheiTrend = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Instantaneous Trend Indicator (iTrend): Trend indicator with nearly zero lag and about the same smoothing as EMA. Trade signals are generated by crossing of the trigger line and iTrend line. Original Code for Tradestation from the book of John F. Ehlers 'Cybernetic Analysis for Stocks and Futures' 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.iTrend iTrend(double alpha)
        {
            return _indicator.iTrend(Input, alpha);
        }

        /// <summary>
        /// Instantaneous Trend Indicator (iTrend): Trend indicator with nearly zero lag and about the same smoothing as EMA. Trade signals are generated by crossing of the trigger line and iTrend line. Original Code for Tradestation from the book of John F. Ehlers 'Cybernetic Analysis for Stocks and Futures' 
        /// </summary>
        /// <returns></returns>
        public Indicator.iTrend iTrend(Data.IDataSeries input, double alpha)
        {
            return _indicator.iTrend(input, alpha);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Instantaneous Trend Indicator (iTrend): Trend indicator with nearly zero lag and about the same smoothing as EMA. Trade signals are generated by crossing of the trigger line and iTrend line. Original Code for Tradestation from the book of John F. Ehlers 'Cybernetic Analysis for Stocks and Futures' 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.iTrend iTrend(double alpha)
        {
            return _indicator.iTrend(Input, alpha);
        }

        /// <summary>
        /// Instantaneous Trend Indicator (iTrend): Trend indicator with nearly zero lag and about the same smoothing as EMA. Trade signals are generated by crossing of the trigger line and iTrend line. Original Code for Tradestation from the book of John F. Ehlers 'Cybernetic Analysis for Stocks and Futures' 
        /// </summary>
        /// <returns></returns>
        public Indicator.iTrend iTrend(Data.IDataSeries input, double alpha)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.iTrend(input, alpha);
        }

    }
}
#endregion
